import io

import requests
import transaction
from BeautifulSoup import BeautifulSoup as bs
from OFS.SimpleItem import SimpleItem
from Products.Reportek import zip_content


class BaseRemoteApplication(SimpleItem):
    def extract_metadata(self, fb):
        fb_status = 'UNKNOWN'
        fb_message = 'N/A'
        f = getattr(fb, 'feedbacktext', None)
        if f:
            soup = bs(f)
            log_sum = soup.find('span', attrs={'id': 'feedbackStatus'})
            if log_sum:
                fb_status = log_sum.get('class', 'UNKNOWN')
                fb_message = log_sum.text
        return fb_status, fb_message

    def add_zip_feedback(self, archive, fb, files, wk, l_file_id, l_ret,
                         job_id, restricted=False):
        """"""
        envelope = self.aq_parent
        feedback_id = '{0}_{1}_{2}'.format(self.app_name, fb, job_id)
        if l_file_id == 'xml':
            l_filename = ' result for: '
        else:
            l_filename = ' result for file %s: ' % l_file_id
        fb_title = ''.join([self.app_name,
                            l_filename,
                            l_ret['SCRIPT_TITLE']])
        envelope.manage_addFeedback(id=feedback_id,
                                    title=fb_title,
                                    activity_id=wk.activity_id,
                                    automatic=1,
                                    document_id=l_file_id,
                                    restricted=restricted)
        feedback_ob = envelope[feedback_id]
        for f in files:
            archive.setcurrentfile(f)
            f_name = envelope.cook_file_id(f)
            if f_name:
                if f_name.endswith('.html'):
                    feedback_ob.feedbacktext = archive.read()
                    feedback_ob.content_type = 'text/html'
                    fb_status, fb_message = self.extract_metadata(feedback_ob)
                    feedback_ob.feedback_status = fb_status
                    if fb_status == 'BLOCKER':
                        wk.blocker = True
                    feedback_ob.message = fb_message
                else:
                    feedback_ob.manage_uploadFeedback(archive, filename=f_name)

    def handle_remote_file(self, url, l_file_id, workitem_id, l_ret, job_id,
                           restricted=False):
        """"""
        wk = getattr(self, workitem_id)
        result = {}
        try:
            r = requests.get(url, allow_redirects=True,
                             headers={'Authorization': self.token},
                             verify=False)
            result['status_code'] = r.status_code
            result['url'] = url
            result['content_type'] = r.headers.get('Content-Type', '')
            result['content_lenght'] = len(r.content)
            if r.status_code == requests.codes.ok:
                from contextlib import closing
                zip = io.BytesIO(r.content)
                with closing(r), zip_content.ZZipFile(zip) as archive:
                    fbs = {}
                    for name in archive.namelist():
                        k = name.split('/')[-1].split('.')[0]
                        if k:
                            if k not in fbs.keys():
                                fbs[k] = [name]
                            else:
                                fbs[k].append(name)
                    for fb in fbs:
                        self.add_zip_feedback(archive, fb, fbs[fb], wk,
                                              l_file_id, l_ret, job_id,
                                              restricted=restricted)
                    transaction.commit()
            else:
                result['content'] = r.content
                wk.addEvent(
                    'Error while downloading results for job #{} from {}. '
                    'Got {} status.'.format(job_id, url, r.status_code))
                wk.failure = True
        except Exception as e:
            # log this
            result['content'] = str(e)
            wk.addEvent(
                'Error while downloading results for job #{} from {}. '
                'Got {} error.'.format(job_id, url, str(e)))
            wk.failure = True

        return result
